@@ -12,12 +12,39 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 import markdown2
 import yaml
 from dataclasses_json import dataclass_json
-from ngwidgets.yamlable import YamlAble
+from ngwidgets.yamlable import YamlAble, lod_storable
 from slugify import slugify
 
 from dcm.svg import SVG, SVGNodeConfig
 
 
+@dataclass_json
+@dataclass
+class RingSpec:
+    """
+    Specification of rings of the donut chart.
+
+    Attributes:
+        text_mode (Optional[str]): The mode of text display on the ring. Default is None.
+        inner_ratio (Optional[float]): The inner radius of the ring, relative to the chart size.
+        outer_ratio (Optional[float]): The outer radius of the ring, relative to the chart size.
+    """
+
+    text_mode: Optional[str] = "empty"
+    inner_ratio: Optional[float] = None
+    outer_ratio: Optional[float] = None
+    levels_visible: Optional[bool] = False
+
+    @property
+    def empty(self) -> bool:
+        empty = (
+            self.inner_ratio is None
+            or self.outer_ratio is None
+            or (self.inner_ratio + self.outer_ratio) == 0.0
+        )
+        return empty
+
+
 @dataclass_json
 @dataclass
 class CompetenceElement:
@@ -30,7 +57,8 @@ class CompetenceElement:
         id (Optional[str]): An optional identifier for the competence element will be set to the name if id is None.
         url (Optional[str]): An optional URL for more information about the competence element.
         description (Optional[str]): An optional description of the competence element.
-        color_code (str): A string representing a color code associated with the competence element.
+        color_code (str): A string representing a (fill) color code associated with the competence element.
+        border_color (str): A string representing the border color to be used e.g. "black" or "#ffffff"
     """
 
     name: str
@@ -39,12 +67,13 @@ class CompetenceElement:
     url: Optional[str] = None
     description: Optional[str] = None
     color_code: Optional[str] = None
+    border_color: Optional[str] = None
 
     def __post_init__(self):
         # Set the id to the the slug of the name if id is None
         if self.id is None:
             # https://pypi.org/project/python-slugify/
-            self.id = slugify(self.name, lowercase=False, regex_pattern=r"[^\w\s\-]")
+            self.id = slugify(self.name, lowercase=False, regex_pattern=r"[^\w\-]")
         if self.short_name is None:
             self.short_name = self.name[:10]
 
@@ -80,6 +109,7 @@ class CompetenceElement:
             id=f"{self.id}",
             url=url,
             fill=self.color_code,
+            color=self.border_color,
             title=self.name,
             comment=comment,
             **kwargs,
@@ -96,6 +126,11 @@ class CompetenceFacet(CompetenceElement):
     This class can include additional properties or methods specific to a competence facet.
     """
 
+    time: Optional[float] = None
+    time_unit: Optional[str] = "h"
+    max_score: Optional[float] = 100.0
+    score_unit: Optional[str] = "%"
+
 
 @dataclass_json
 @dataclass
@@ -148,9 +183,14 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
     Represents the entire structure of competencies, including various aspects and levels.
 
     Attributes:
-        competence_aspects (List[CompetenceAspect]): A list of CompetenceAspect objects.
-        competence_levels (List[CompetenceLevel]): A list of CompetenceLevel objects representing the different levels in the competence hierarchy.
+        lookup_url (Optional[str]): Optional URL for additional information.
+        total_levels (int): Total number of levels in the competence hierarchy.
+        stacked_levels (Optional[bool]): Indicates whether the levels are stacked.
+        aspects (List[CompetenceAspect]): A list of CompetenceAspect objects.
+        levels (List[CompetenceLevel]): A list of CompetenceLevel objects.
         element_names (Dict[str, str]): A dictionary holding the names for tree, aspects, facets, and levels.  The key is the type ("tree", "aspect", "facet", "level").
+        ring_specs (Dict[str, RingSpec]): Specifications for the rings in the donut chart.
+        total_elements (Dict[str, int]): A dictionary holding the total number of elements for each type (aspects, areas, facets).
     """
 
     lookup_url: Optional[str] = None
@@ -159,7 +199,7 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
     aspects: List[CompetenceAspect] = field(default_factory=list)
     levels: List[CompetenceLevel] = field(default_factory=list)
     element_names: Dict[str, str] = field(default_factory=dict)
-    relative_radius: Dict[str, Tuple[float, float]] = field(default_factory=dict)
+    ring_specs: Dict[str, RingSpec] = field(default_factory=dict)
     total_elements: Dict[str, int] = field(default_factory=dict)
 
     def __post_init__(self):
@@ -169,6 +209,29 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
         super().__post_init__()
         self.total_elements = {"aspects": 0, "areas": 0, "facets": 0}
         self.update_paths()
+        self.calculate_ring_specs("empty")
+
+    def calculate_ring_specs(self, text_mode: str):
+        """
+        calculate the ring specifications
+        """
+        inner_ratio = 0.0
+        circle_ratio = 1 / (self.total_levels * 2 + 1)
+        outer_ratio = circle_ratio
+        if not "tree" in self.ring_specs:
+            self.ring_specs["tree"] = RingSpec(
+                text_mode=text_mode, inner_ratio=0.0, outer_ratio=circle_ratio
+            )
+        # loop over ring levels
+        for rl in ["aspect", "area", "facet"]:
+            inner_ratio = outer_ratio
+            outer_ratio = outer_ratio + circle_ratio * 2
+            if not rl in self.ring_specs:
+                self.ring_specs[rl] = RingSpec(
+                    text_mode=text_mode,
+                    inner_ratio=inner_ratio,
+                    outer_ratio=outer_ratio,
+                )
 
     def update_paths(self):
         """
@@ -201,7 +264,7 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
 
     @classmethod
     def required_keys(cls) -> Tuple:
-        keys = {"name", "id", "url", "description", "element_names"}
+        keys = {"name", "id", "url", "description", "aspects"}
         return keys
 
     def lookup_by_path(
@@ -310,15 +373,13 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
             box_width,
             box_height,
         )
-
+        max_name_width = 5
+        if self.levels:
+            max_name_width = max(
+                svg.get_text_width(level.name) for level in self.levels
+            )
         # Calculate the x position for the aspect legend based on the width of the level legend
-        x_aspect_start = (
-            x_start
-            + box_width
-            + padding
-            + max(svg.get_text_width(level.name) for level in self.levels)
-            + padding
-        )
+        x_aspect_start = x_start + box_width + padding + max_name_width + padding
 
         # Add the competence aspect legend
         aspect_items = [(aspect.color_code, aspect.name) for aspect in self.aspects]
@@ -375,8 +436,7 @@ class Achievement:
         return parts[3] if len(parts) > 3 else None
 
 
-@dataclass_json
-@dataclass
+@lod_storable
 class Learner:
     """
     A learner with achievements.
@@ -392,9 +452,11 @@ class Learner:
 
     def __post_init__(self):
         self.achievements_by_path = {}
+        self.achievement_indices_by_path = {}
         if self.achievements:
-            for achievement in self.achievements:
+            for index, achievement in enumerate(self.achievements):
                 self.achievements_by_path[achievement.path] = achievement
+                self.achievement_indices_by_path[achievement.path] = index
 
     @classmethod
     def required_keys(cls):
@@ -413,9 +475,18 @@ class Learner:
         )
         return file_name
 
+    def get_achievement_index(self, path) -> int:
+        a_index = self.achievement_indices_by_path.get(path)
+        return a_index
+
     def add_achievement(self, new_achievement):
+        """
+        Add a new achievement for the learner.
+        """
         self.achievements.append(new_achievement)
+        index = len(self.achievements) - 1
         self.achievements_by_path[new_achievement.path] = new_achievement
+        self.achievement_indices_by_path[new_achievement.path] = index
 
     def get_competence_tree_ids(self) -> List[str]:
         """
