@@ -3,18 +3,15 @@ Created on 2023-11-06
 
 @author: wf
 """
-import json
 import os
 import uuid
-from dataclasses import dataclass
 from typing import List, Optional
 from urllib.parse import urlparse
 
-import yaml
 from fastapi import HTTPException
 from fastapi.responses import HTMLResponse
 from ngwidgets.file_selector import FileSelector
-from ngwidgets.input_webserver import InputWebserver
+from ngwidgets.input_webserver import InputWebserver, InputWebSolution
 from ngwidgets.webserver import WebserverConfig
 from nicegui import Client, app, ui
 from pydantic import BaseModel
@@ -22,10 +19,10 @@ from pydantic import BaseModel
 from dcm.dcm_assessment import Assessment
 from dcm.dcm_chart import DcmChart
 from dcm.dcm_core import CompetenceTree, DynamicCompetenceMap, Learner
+from dcm.dcm_web import RingSpecsView
 from dcm.svg import SVG, SVGConfig
 from dcm.version import Version
 
-
 class SVGRenderRequest(BaseModel):
     """
     A request for rendering an SVG.
@@ -36,32 +33,12 @@ class SVGRenderRequest(BaseModel):
         markup (str): The format of the definition ('json' or 'yaml').
         config (SVGConfig): Optional configuration for SVG rendering. Defaults to None, which uses default settings.
     """
-
     name: str
     definition: str
     markup: str
+    text_mode: Optional[str] = "empty"
     config: Optional[SVGConfig] = None
 
-
-@dataclass
-class ServerConfig:
-    storage_secret: str
-    storage_path: str
-
-    @classmethod
-    def from_yaml(cls, yaml_path: str):
-        if not os.path.exists(yaml_path):
-            default_storage_path = os.path.join(
-                os.path.expanduser("~"), ".dcm", "storage"
-            )
-            # Create the directory if it does not exist
-            os.makedirs(default_storage_path, exist_ok=True)
-            return cls(str(uuid.uuid4()), default_storage_path)
-        with open(yaml_path, "r") as file:
-            config_data = yaml.safe_load(file)
-        return cls(**config_data)
-
-
 class DynamicCompentenceMapWebServer(InputWebserver):
     """
     server to supply Dynamic Competence Map Visualizations
@@ -72,11 +49,16 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         """
         get the configuration for this Webserver
         """
-        copy_right = "(c)2023-2024 Wolfgang Fahl"
+        copy_right = ""
         config = WebserverConfig(
-            copy_right=copy_right, version=Version(), default_port=8885
+            short_name="dcm",
+            copy_right=copy_right, 
+            version=Version(), 
+            default_port=8885
         )
-        return config
+        server_config=WebserverConfig.get(config)
+        server_config.solution_class=DcmSolution
+        return server_config
 
     def __init__(self):
         """Constructs all the necessary attributes for the WebServer object."""
@@ -84,18 +66,8 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             self, config=DynamicCompentenceMapWebServer.get_config()
         )
         self.examples = DynamicCompetenceMap.get_examples(markup="yaml")
-        self.dcm = None
-        self.container = None
-        self.learner = None
-        self.assessment = None
-        self.text_mode = "none"
-        config_path = os.path.join(os.environ["HOME"], ".dcm/config.yaml")
-        self.server_config = ServerConfig.from_yaml(config_path)
-
-        @app.get("/learner/{learner_slug}")
-        async def show_learner(learner_slug: str):
-            return await self.assess_learner_by_slug(learner_slug)
-
+  
+        # FastAPI endpoints
         @app.post("/svg/")
         async def render_svg(svg_render_request: SVGRenderRequest) -> HTMLResponse:
             """
@@ -176,6 +148,26 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             path = f"{tree_id}"
             return await self.show_description(path)
 
+        # nicegui RESTFul endpoints
+        @ui.page("/learner/{learner_slug}")
+        async def show_learner(client: Client, learner_slug: str):
+            return await self.page(client, DcmSolution.assess_learner_by_slug,learner_slug)
+        
+    async def render_svg(self, svg_render_request: SVGRenderRequest) -> HTMLResponse:
+            """
+            render the given request
+            """
+            r = svg_render_request
+            dcm = DynamicCompetenceMap.from_definition_string(
+                r.name, r.definition, content_class=CompetenceTree, markup=r.markup
+            )
+            dcm_chart = DcmChart(dcm)
+            svg_markup = dcm_chart.generate_svg_markup(
+                config=r.config, with_java_script=True, text_mode=r.text_mode
+            )
+            response = HTMLResponse(content=svg_markup)
+            return response
+        
     async def show_description(self, path: str = None) -> HTMLResponse:
         """
         Show the HTML description of a specific
@@ -205,20 +197,40 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             msg = f"unknown competence tree {tree_id}"
             raise HTTPException(status_code=404, detail=msg)
 
-    async def render_svg(self, svg_render_request: SVGRenderRequest) -> HTMLResponse:
+    def configure_run(self):
         """
-        render the given request
+        configure the allowed urls
         """
-        r = svg_render_request
-        dcm = DynamicCompetenceMap.from_definition_string(
-            r.name, r.definition, content_class=CompetenceTree, markup=r.markup
-        )
-        dcm_chart = DcmChart(dcm)
-        svg_markup = dcm_chart.generate_svg_markup(
-            config=r.config, with_java_script=True, text_mode=self.text_mode
-        )
-        response = HTMLResponse(content=svg_markup)
-        return response
+        InputWebserver.configure_run(self)
+        self.allowed_urls = [
+            # "https://raw.githubusercontent.com/JuanIrache/DJI_SRT_Parser/master/samples/",
+            # "https://raw.githubusercontent.com/JuanIrache/dji-srt-viewer/master/samples/",
+            # "https://cycle.travel/gpx/",
+            # "https://cycle.travel/map/journey/",
+            DynamicCompetenceMap.examples_path(),
+            self.root_path,
+        ]
+        pass
+        
+class DcmSolution(InputWebSolution):
+    """
+    the Dynamic Competence Map solution
+    """
+    
+    def __init__(self, webserver: DynamicCompentenceMapWebServer, client: Client):
+        """
+        Initialize the solution
+
+        Calls the constructor of the base solution
+        Args:
+            webserver (DynamicCompotenceMapWebServer): The webserver instance associated with this context.
+            client (Client): The client instance this context is associated with.
+        """
+        super().__init__(webserver, client)  # Call to the superclass constructor
+        self.dcm = None
+        self.learner=None
+        self.assessment = None
+        self.text_mode = "empty"
 
     def get_basename_without_extension(self, url) -> str:
         # Parse the URL to get the path component
@@ -228,6 +240,42 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         # Split the base name and extension and return just the base name
         return os.path.splitext(basename)[0]
 
+    def save_session_state(self) -> None:
+        """
+        Save the current session state to app.storage.user.
+        """
+        learner_id=self.learner.learner_id if self.learner else None
+        app.storage.user['learner_id'] = learner_id
+        app.storage.user['assessment'] = self.assessment is not None
+
+    def get_learner_file_path(self, learner_slug: str) -> str:
+        """
+        Get the file path for a learner's JSON file based on the learner's slug.
+
+        Args:
+            learner_slug (str): The unique slug of the learner.
+
+        Returns:
+            str: The file path for the learner's JSON file.
+        """
+        return os.path.join(self.config.storage_path, f"{learner_slug}.json")
+
+    def load_learner(self, learner_slug: str) -> None:
+        """
+        Load a learner from a JSON file based on the learner's slug.
+
+        Args:
+            learner_slug (str): The unique slug of the learner.
+
+        Raises:
+            FileNotFoundError: If the learner file does not exist.
+        """
+        learner_file = self.get_learner_file_path(learner_slug)
+        if not os.path.exists(learner_file):
+            raise FileNotFoundError(f"Learner file not found: {learner_file}")
+        self.learner = Learner.load_from_json_file(learner_file)
+        self.save_session_state()
+
     async def render(self, _click_args=None):
         """
         Renders the json content as an SVG visualization
@@ -239,7 +287,7 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             input_source = self.input
             if input_source:
                 name = self.get_basename_without_extension(input_source)
-                with self.container:
+                with self.content_div:
                     ui.notify(f"rendering {name}")
                 definition = self.do_read_input(input_source)
                 # Determine the format based on the file extension
@@ -253,13 +301,14 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                 )
                 self.render_item(item)
         except Exception as ex:
-            self.handle_exception(ex, self.do_trace)
+            self.handle_exception(ex)
 
     def render_item(self, item):
         if isinstance(item, DynamicCompetenceMap):
             self.render_dcm(item)
         else:
             self.learner = item
+            self.save_session_state()
             self.assess(item)
 
     def render_dcm(
@@ -288,7 +337,8 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                 self.assessment = None
                 self.learner = None
             self.dcm = dcm
-            self.assess_state(True)
+            self.ringspecs_view.update_rings(dcm.competence_tree)
+            self.assess_state()
             dcm_chart = DcmChart(dcm)
             svg_markup = dcm_chart.generate_svg_markup(
                 learner=learner,
@@ -301,9 +351,19 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             self.svg_view.content = (svg_markup,)
             self.svg_view.update()
         except Exception as ex:
-            self.handle_exception(ex, self.do_trace)
+            self.handle_exception(ex)
 
     def prepare_ui(self):
+        """
+        prepare the user interface
+        """
+        self.user_id = app.storage.browser['id']  
+        self.prepare_svg()
+        
+    def prepare_svg(self):
+        """
+        prepare the SVG / javascript display
+        """
         config = SVGConfig(with_popup=True)
         self.svg = SVG(config=config)
         java_script = self.svg.get_java_script()
@@ -312,23 +372,20 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         ui.add_head_html(java_script)
 
     def show_ui(self):
-        with ui.element("div").classes("w-full") as self.container:
+        """
+        show the ui
+        """
+        with self.content_div:
             with ui.splitter() as splitter:
                 with splitter.before:
                     with ui.grid(columns=2).classes("w-full") as self.left_selection:
                         extensions = {"json": ".json", "yaml": ".yaml"}
                         self.example_selector = FileSelector(
-                            path=self.root_path,
+                            path=self.webserver.root_path,
                             extensions=extensions,
                             handler=self.read_and_optionally_render,
                         )
-                        selection = ["none", "curved", "horizontal", "angled"]
-                        self.add_select(
-                            "text",
-                            selection,
-                            value=self.text_mode,
-                            on_change=self.on_text_mode_change,
-                        )
+                        self.ringspecs_view = RingSpecsView(self)
                     with ui.grid(columns=1).classes("w-full") as self.left_grid:
                         with ui.row() as self.input_row:
                             self.input_input = ui.input(
@@ -359,12 +416,12 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                 with splitter.after:
                     self.svg_view = ui.html("")
 
-    async def home(self, _client: Client):
+    async def home(self,):
         """Generates the home page with a selection of examples and
         svg display
         """
         await self.setup_content_div(self.show_ui)
-        self.assess_state(False)
+        self.assess_state()
 
     def assess_learner(self, dcm, learner):
         """
@@ -375,22 +432,28 @@ class DynamicCompentenceMapWebServer(InputWebserver):
             learner(Learner): the learner to get the self assessment for
 
         """
-        if self.container:
-            with self.container:
-                if self.assessment is not None:
-                    self.assessment.reset(dcm=dcm, learner=learner)
-                else:
-                    with self.left_grid:
-                        with ui.row() as self.assessment_row:
-                            self.assessment = Assessment(self, dcm=dcm, learner=learner)
-                self.assessment.update_achievement_view()
+        if not self.content_div:
+            return
+        with self.content_div:
+            if self.assessment is not None:
+                self.assessment.reset(dcm=dcm, learner=learner)
+            else:
+                with self.left_grid:
+                    with ui.row() as self.assessment_row:
+                        self.assessment = Assessment(self, dcm=dcm, learner=learner)
+            self.assessment.step(0)
 
     def new_assess(self):
         """
         run a new  assessment for a new learner
         """
-        self.learner = Learner(learner_id=f"{uuid.uuid4()}")
-        self.assess_learner(self.dcm, self.learner)
+        try: 
+            learner_id=f"{uuid.uuid4()}"
+            self.learner=Learner(learner_id)
+            self.save_session_state()
+            self.assess_learner(self.dcm, self.learner)
+        except Exception as ex:
+            self.handle_exception(ex)
 
     async def assess_learner_by_slug(self, learner_slug: str):
         """
@@ -404,22 +467,14 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         """
 
         def show():
-            learner_file = os.path.join(
-                self.server_config.storage_path, f"{learner_slug}.json"
-            )
-            if not os.path.exists(learner_file):
-                raise HTTPException(status_code=404, detail="Learner not found")
             try:
-                with open(learner_file, "r") as file:
-                    learner_data = json.load(file)
-                    learner = Learner.from_dict(learner_data)
-            except Exception as e:
-                # Handle any exceptions related to file reading or JSON parsing
-                raise HTTPException(status_code=500, detail=str(e))
-                pass
-            self.assess(learner)
+                self.show_ui()
+                self.load_learner(learner_slug)
+                self.assess(self.learner)
+            except Exception as ex:
+                self.handle_exception(ex)
 
-        await self.setup_content_div(show())
+        await self.setup_content_div(show)
 
     def assess(self, learner: Learner, tree_id: str = None):
         """
@@ -436,21 +491,36 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                     f"There must be exactly one competence tree referenced but there are: {tree_ids}"
                 )
             tree_id = tree_ids[0]
-        if not tree_id in self.examples:
+        if not tree_id in self.webserver.examples:
             raise Exception(f"invalid competence tree_id {tree_id}")
-        dcm = self.examples[tree_id]
+        dcm = self.webserver.examples[tree_id]
         # assess_learner will render ...
         # self.render_dcm(dcm,learner=learner)
         self.assess_learner(dcm, learner)
 
-    def assess_state(self, state: bool):
-        if state:
+    def on_update_ringspecs(self):
+        if self.learner:
+            self.render_item(self.learner)
+        else:
+            self.render_item(self.dcm)
+
+    def assess_state(self):
+        """
+        save the session state and reflect
+        in the UI
+        """
+        self.save_session_state()
+        if self.dcm and self.learner is None:
+            # allow creating a new learner
             self.assessment_button.enable()
         else:
+            # the assessment is already on
             self.assessment_button.disable()
-        if self.learner:
+        if self.assessment is not None:
+                # downloading is possible
             self.download_button.enable()
         else:
+            # downloading is not possible - we have n learner
             self.download_button.disable()
 
     async def download(self, _args):
@@ -458,7 +528,7 @@ class DynamicCompentenceMapWebServer(InputWebserver):
         allow downloading the assessment result
         """
         try:
-            with self.container:
+            with self.content_div:
                 if not self.assessment:
                     ui.notify("no active learner assessment")
                     return
@@ -466,31 +536,4 @@ class DynamicCompentenceMapWebServer(InputWebserver):
                 ui.notify(f"downloading {json_path}")
                 ui.download(json_path)
         except Exception as ex:
-            self.handle_exception(ex, self.do_trace)
-
-    async def on_text_mode_change(self, args):
-        """
-        handle changes in the text_mode
-        """
-        new_text_mode = args.value
-        if new_text_mode != self.text_mode:
-            self.text_mode = new_text_mode
-            if self.learner:
-                self.render_item(self.learner)
-            else:
-                self.render_item(self.dcm)
-
-    def configure_run(self):
-        """
-        configure the allowed urls
-        """
-        self.allowed_urls = [
-            # "https://raw.githubusercontent.com/JuanIrache/DJI_SRT_Parser/master/samples/",
-            # "https://raw.githubusercontent.com/JuanIrache/dji-srt-viewer/master/samples/",
-            # "https://cycle.travel/gpx/",
-            # "https://cycle.travel/map/journey/",
-            DynamicCompetenceMap.examples_path(),
-            self.root_path,
-        ]
-        self.args.storage_secret = self.server_config.storage_secret
-        pass
+            self.handle_exception(ex)
