@@ -59,6 +59,8 @@ class SVGNode:
     color: Optional[
         str
     ] = None  # Color of font or stroke use default color of config if None
+    stroke_width: float = 1.0  # Stroke width of the polygon lines
+    opacity: float = 0.5  # Opacity of fill e.g. for polygons
     fill: Optional[str] = "black"  # Fill color for the segment
     title: Optional[str] = None  # Tooltip
     comment: Optional[str] = None
@@ -130,6 +132,16 @@ class Arc:
     middle_y: Optional[float] = None
 
 
+@dataclass
+class Polygon(SVGNode):
+    """
+    A polygon representing a series of data points in a radar chart.
+    """
+
+    # List of points (x, y) forming the polygon
+    points: List[Tuple[float, float]] = field(default_factory=list)
+
+
 @dataclass
 class DonutSegment(SVGNode):
     """
@@ -143,6 +155,7 @@ class DonutSegment(SVGNode):
     outer_radius: float = 0.0
     start_angle: Optional[float] = 0.0
     end_angle: Optional[float] = 360.0
+    text_mode: Optional[str] = None
 
     @property
     def large_arc_flag(self) -> str:
@@ -176,7 +189,7 @@ class DonutSegment(SVGNode):
 
     def get_arc(self, radial_offset: float = 0.5) -> Arc:
         """
-        Get the Arc for the givne radial offset
+        Get the Arc for the given radial offset
 
         Args:
             radial_offset(float): e.g. 0.0 - inner 1.0 outer 0.5 middle
@@ -242,7 +255,7 @@ class SVG:
             f"{self.indent}<style>\n"
             f"{self.indent * 2}.hoverable {{ cursor: pointer; fill-opacity: 1; stroke: black; stroke-width: 0.5; }}\n"
             f"{self.indent * 2}.hoverable:hover {{ fill-opacity: 0.7; }}\n"
-            f"{self.indent * 2}.selected {{ fill-opacity: 0.5; stroke: blue; stroke-width: 1.5;}}\n"
+            f"{self.indent * 2}.selected {{ fill-opacity: 0.5; stroke: blue !important; stroke-width: 1.5;}}\n"
             f"{self.indent * 2}.noclick {{ pointer-events: none; }}\n"  # style for non-clickable text
         )
 
@@ -412,6 +425,43 @@ class SVG:
         rect = f'<rect x="{x}" y="{y}" width="{width}" height="{height}" fill="{color}" />\n'
         self.add_element(rect, indent_level=indent_level)
 
+    def add_polygon(self, polygon: Polygon, indent_level: int = 1):
+        """
+        Add a polygon to the SVG.
+
+        Args:
+            polygon (Polygon): The polygon to add to the SVG.
+            indent_level (int): Indentation level for the element.
+        """
+        points_str = " ".join(f"{x},{y}" for x, y in polygon.points)
+        fill_color = polygon.fill if polygon.fill else self.config.default_color
+        opacity = polygon.opacity
+        stroke_color = polygon.color if polygon.color else "black"
+        stroke_width = polygon.stroke_width
+
+        polygon_element = (
+            f'<polygon points="{points_str}" '
+            f'stroke="{stroke_color}" stroke-width="{stroke_width}" '
+            f'fill="{fill_color}" fill-opacity="{opacity}" />\n'
+        )
+
+        # If a title (tooltip) is provided, create a title element
+        title_element = (
+            f"<title>{html.escape(polygon.title)}</title>\n" if polygon.title else ""
+        )
+
+        # Combine polygon and title into one string without adding indentation here
+        group_content = f"{polygon_element}{title_element}"
+
+        # Use add_group to add the polygon with proper indentation and optional comment
+        self.add_group(
+            group_content,
+            group_id=polygon.id,
+            group_class="hoverable",  # You might want to adjust this or make it configurable
+            indent_level=indent_level,
+            comment=polygon.comment,
+        )
+
     def add_legend_column(
         self,
         items: List[Tuple[str, str]],
@@ -543,14 +593,14 @@ class SVG:
             segment(DonutSegment)
         """
         color = config.fill if config.fill else self.config.default_color
-
-        if color is None:
-            color = self.config.default_color
+        stroke_color = config.color if config.color else "black"
 
         path_str = self.get_donut_path(segment)
 
         # Assemble the path and title elements
-        path_element = f'<path d="{path_str}" fill="{color}" />\n'
+        path_element = (
+            f'<path d="{path_str}" stroke="{stroke_color}" fill="{color}" />\n'
+        )
         if config.title:
             escaped_title = html.escape(config.title)  # Escape special characters
 
